@@ -4,12 +4,14 @@
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
 # License:: You can redistribute it and/or modify it under the same term as Ruby.
-# Revision:: $Id: generic.rb 56878 2016-11-22 23:44:51Z kazu $
+# Revision:: $Id: generic.rb 61225 2017-12-14 06:30:22Z naruse $
 #
 # See URI for general documentation
 #
 
 require 'uri/common'
+autoload :IPSocket, 'socket'
+autoload :IPAddr, 'ipaddr'
 
 module URI
 
@@ -271,7 +273,7 @@ module URI
     #
     # Portion of the path that does make use of the slash '/'.
     # The path typically refers to the absolute path and the opaque part.
-    #  (see RFC2396 Section 3 and 5.2)
+    # (See RFC2396 Section 3 and 5.2.)
     #
     attr_reader :opaque
 
@@ -643,9 +645,9 @@ module URI
     # This method is same as URI::Generic#host except
     # brackets for IPv6 (and future IP) addresses are removed.
     #
-    # u = URI("http://[::1]/bar")
-    # p u.hostname      #=> "::1"
-    # p u.host          #=> "[::1]"
+    #   u = URI("http://[::1]/bar")
+    #   p u.hostname      #=> "::1"
+    #   p u.host          #=> "[::1]"
     #
     def hostname
       v = self.host
@@ -657,10 +659,10 @@ module URI
     # This method is same as URI::Generic#host= except
     # the argument can be bare IPv6 address.
     #
-    # u = URI("http://foo/bar")
-    # p u.to_s                  #=> "http://foo/bar"
-    # u.hostname = "::1"
-    # p u.to_s                  #=> "http://[::1]/bar"
+    #   u = URI("http://foo/bar")
+    #   p u.to_s                  #=> "http://foo/bar"
+    #   u.hostname = "::1"
+    #   p u.to_s                  #=> "http://[::1]/bar"
     #
     # If the argument seems IPv6 address,
     # it is wrapped by brackets.
@@ -977,7 +979,7 @@ module URI
     # returns an Array of the path split on '/'
     #
     def split_path(path)
-      path.split(%r{/+}, -1)
+      path.split("/", -1)
     end
     private :split_path
 
@@ -1152,8 +1154,8 @@ module URI
         return dst.dup
       end
 
-      src_path = src.scan(%r{(?:\A|[^/]+)/})
-      dst_path = dst.scan(%r{(?:\A|[^/]+)/?})
+      src_path = src.scan(%r{[^/]*/})
+      dst_path = dst.scan(%r{[^/]*/?})
 
       # discard same parts
       while !dst_path.empty? && dst_path.first == src_path.first
@@ -1293,7 +1295,17 @@ module URI
     end
 
     #
-    # Returns normalized URI
+    # Returns normalized URI.
+    #
+    #   require 'uri'
+    #
+    #   URI("HTTP://my.EXAMPLE.com").normalize
+    #   #=> #<URI::HTTP http://my.example.com/>
+    #
+    # Normalization here means:
+    #
+    # * scheme and host are converted to lowercase,
+    # * an empty path component is set to "/".
     #
     def normalize
       uri = dup
@@ -1329,7 +1341,7 @@ module URI
       if @opaque
         str << @opaque
       else
-        if @host
+        if @host || %w[file postgres].include?(@scheme)
           str << '//'
         end
         if self.userinfo
@@ -1357,7 +1369,7 @@ module URI
     end
 
     #
-    # Compares to URI's
+    # Compares two URIs
     #
     def ==(oth)
       if self.class == oth.class
@@ -1438,8 +1450,8 @@ module URI
     #
     # == Description
     #
-    #  attempt to parse other URI +oth+
-    #  return [parsed_oth, self]
+    # attempts to parse other URI +oth+,
+    # returns [parsed_oth, self]
     #
     # == Usage
     #
@@ -1505,7 +1517,7 @@ module URI
       elsif name == 'http_proxy'
         unless proxy_uri = env[name]
           if proxy_uri = env[name.upcase]
-            warn 'The environment variable HTTP_PROXY is discouraged.  Use http_proxy.'
+            warn 'The environment variable HTTP_PROXY is discouraged.  Use http_proxy.', uplevel: 1
           end
         end
       else
@@ -1517,7 +1529,6 @@ module URI
       end
 
       if self.hostname
-        require 'socket'
         begin
           addr = IPSocket.getaddress(self.hostname)
           return nil if /\A127\.|\A::1\z/ =~ addr
@@ -1527,23 +1538,26 @@ module URI
 
       name = 'no_proxy'
       if no_proxy = env[name] || env[name.upcase]
-        no_proxy.scan(/(?!\.)([^:,\s]+)(?::(\d+))?/) {|host, port|
-          if (!port || self.port == port.to_i)
-            if /(\A|\.)#{Regexp.quote host}\z/i =~ self.host
-              return nil
-            else
-              require 'ipaddr'
-              return nil if
-                begin
-                  IPAddr.new(host)
-                rescue IPAddr::InvalidAddressError
-                  next
-                end.include?(self.host)
-            end
-          end
-        }
+        return nil unless URI::Generic.use_proxy?(self.hostname, addr, self.port, no_proxy)
       end
       URI.parse(proxy_uri)
     end
+
+    def self.use_proxy?(hostname, addr, port, no_proxy) # :nodoc:
+      no_proxy.scan(/(?!\.)([^:,\s]+)(?::(\d+))?/) {|p_host, p_port|
+        if !p_port || port == p_port.to_i
+          if /(\A|\.)#{Regexp.quote p_host}\z/i =~ hostname
+            return false
+          elsif addr
+            begin
+              return false if IPAddr.new(p_host).include?(addr)
+            rescue IPAddr::InvalidAddressError
+              next
+            end
+          end
+        end
+      }
+      true
+    end
   end
 end
